<!-- templates/alerts.html -->
{% if request.app.alerts %}
    {% set alerts = request.app.alerts %}
    {#    {{ alerts }}#}
    <div id="alert_bar">
    {%  include 'alerts.html' with context %}
{#        <div class="wrapper">#}
{#            {% for alert in request.app.alerts.alert %}#}
{#                <div class="alert alert__{{ alert.type }}">#}
{#                    <strong>{{ alert.type }}: </strong> {{ alert.message }} {{ "ErrorCode: " + str(alert.code) if alert.code else "" }}#}
{#                </div>#}
{#            {% endfor %}#}
{#        </div>#}
    </div>
{% endif %}



